@@ -22,81 +22,22 @@
 
 static GdkPixbuf *button_load_pixbuf (const char  *file,
 				      char       **error);
+static GdkPixbuf * get_missing (int preffered_size);
 
 enum {
 	PROP_0,
+	PROP_ACTIVATABLE,
 	PROP_HAS_ARROW,
-	PROP_ORIENT,
+	PROP_DND_HIGHLIGHT,
+	PROP_ORIENTATION,
 	PROP_ICON_NAME,
-	PROP_STOCK_ID,
+	PROP_STOCK_ID
 };
 
 #define BUTTON_WIDGET_DISPLACEMENT 2
 
 static GObjectClass *parent_class;
 
-static void
-translate_to(GtkWidget *from, GtkWidget *to, int *x, int *y)
-{
-	while (from != to) {
-		if (!GTK_WIDGET_NO_WINDOW (from)) {
-			*x += MAX (from->allocation.x, 0);
-			*y += MAX (from->allocation.y, 0);
-		}
-		from = from->parent;
-	}
-}
-
-static void
-calculate_overlay_geometry (PanelWidget *panel,
-			    GtkWidget   *parent,
-			    GtkWidget   *applet,
-			    int         *x,
-			    int         *y,
-			    int         *w,
-			    int         *h)
-{
-	*x = applet->allocation.x;
-	*y = applet->allocation.y;
-	*w = applet->allocation.width;
-	*h = applet->allocation.height;
-
-	translate_to (GTK_WIDGET(panel), parent, x, y);
-
-	if(panel->orient == GTK_ORIENTATION_HORIZONTAL) {
-		if (applet->allocation.x > panel->size) {
-			*x = parent->requisition.width + 1;
-			*y = parent->requisition.height + 1;
-			return;
-		}
-
-		*y = 0;
-		/* we use the requisition, since allocation might have not
-		   yet happened if we are inside the allocation, anyway
-		   they are the same for basep */
-		if(*h < parent->requisition.height)
-			*h = parent->requisition.height;
-
-		if ((*w + applet->allocation.x) > panel->size) {
-			*w = panel->size - applet->allocation.x;
-		}
-	} else {
-		if (applet->allocation.y > panel->size) {
-			*x = parent->requisition.width + 1;
-			*y = parent->requisition.height + 1;
-			return;
-		}
-
-		*x = 0;
-		if(*w < parent->requisition.width)
-			*w = parent->requisition.width;
-
-		if ((*h + applet->allocation.y) > panel->size) {
-			*h = panel->size - applet->allocation.y;
-		}
-	}
-}
-
 /* colorshift a pixbuf */
 static void
 do_colorshift (GdkPixbuf *dest, GdkPixbuf *src, int shift)
@@ -161,27 +102,19 @@ button_widget_realize(GtkWidget *widget)
 	GdkWindowAttr attributes;
 	gint attributes_mask;
 	GtkButton *button;
-	PanelWidget *panel;
-	GtkWidget *parent;
-	int x,y,w,h;
 
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (BUTTON_IS_WIDGET (widget));
 
-	panel  = PANEL_WIDGET(widget->parent);
-	parent = GTK_WIDGET (panel->toplevel);
-
-	calculate_overlay_geometry(panel, parent, widget, &x, &y, &w, &h);
-
 	button = GTK_BUTTON (widget);
 
 	GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
 
 	attributes.window_type = GDK_WINDOW_CHILD;
-	attributes.x = x;
-	attributes.y = y;
-	attributes.width = w;
-	attributes.height = h;
+	attributes.x = widget->allocation.x;
+	attributes.y = widget->allocation.y;
+	attributes.width = widget->allocation.width;
+	attributes.height = widget->allocation.height;
 	attributes.wclass = GDK_INPUT_ONLY;
 	attributes.event_mask = (GDK_BUTTON_PRESS_MASK |
 				 GDK_BUTTON_RELEASE_MASK |
@@ -192,10 +125,10 @@ button_widget_realize(GtkWidget *widget)
 				 GDK_LEAVE_NOTIFY_MASK);
 	attributes_mask = GDK_WA_X | GDK_WA_Y;
 
-	widget->window = gtk_widget_get_parent_window(widget);
+	widget->window = gtk_widget_get_parent_window (widget);
 	g_object_ref (G_OBJECT (widget->window));
       
-	button->event_window = gdk_window_new (parent->window,
+	button->event_window = gdk_window_new (gtk_widget_get_parent_window (widget),
 					       &attributes,
 					       attributes_mask);
 	gdk_window_set_user_data (button->event_window, widget);
@@ -204,24 +137,26 @@ button_widget_realize(GtkWidget *widget)
 }
 
 static void
-button_widget_parent_set (GtkWidget *widget,
-			  GtkWidget *previous_parent)
+button_widget_unrealize (GtkWidget *widget)
 {
+	GtkButton *button;
+	PanelWidget *panel;
 	GtkWidget *parent;
-	int        x, y, w, h;
 
+	g_return_if_fail (widget != NULL);
 	g_return_if_fail (BUTTON_IS_WIDGET (widget));
 
-	if (!GTK_WIDGET_REALIZED (widget)|| !widget->parent)
-		return;
+	panel  = PANEL_WIDGET (widget->parent);
+	parent = GTK_WIDGET (panel->toplevel);
+	button = GTK_BUTTON (widget);
 
-	parent = GTK_WIDGET (PANEL_WIDGET (widget->parent)->toplevel);
+	if (button->event_window != NULL) {
+		gdk_window_set_user_data (button->event_window, NULL);
+		gdk_window_destroy (button->event_window);
+		button->event_window = NULL;
+	}
 
-	calculate_overlay_geometry (
-		PANEL_WIDGET (widget->parent), parent, widget, &x, &y, &w, &h);
-
-	gdk_window_reparent (
-		GTK_BUTTON (widget)->event_window, parent->window, x, y);
+	GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
 }
 
 static void
@@ -247,40 +182,46 @@ button_widget_load_pixbuf_and_scale (ButtonWidget *button)
 	int    width;
 	int    height;
 
-	if (!button->pixbuf) {
-		g_assert (!button->filename || !button->stock_id);
+	if (button->size <= 1)
+		return;
 
+	if (button->pixbuf == NULL) {
 		if (!button->filename && !button->stock_id)
 			return;
 
-		if (button->stock_id)
-			button->pixbuf = gtk_widget_render_icon (
-						GTK_WIDGET (button),
-						button->stock_id,
-						(GtkIconSize) -1,
-						NULL);
-		else {
+		if (button->filename != NULL) {
 			char *error = NULL;
 
 			button->pixbuf = button_load_pixbuf (button->filename, &error);
 			if (error) {
 				panel_error_dialog (gdk_screen_get_default (),
 						    "cannot_load_pixbuf",
-						    _("Failed to load image %s\n\n"
-						    "Details: %s"),
+						    _("Failed to load image %s"),
+						    "%s",
 						    button->filename,
 						    error);
 				g_free (error);
 			}
 		}
 
+		if (button->pixbuf == NULL &&
+		    button->stock_id != NULL)
+			button->pixbuf = gtk_widget_render_icon (
+						GTK_WIDGET (button),
+						button->stock_id,
+						(GtkIconSize) -1,
+						NULL);
+
+/* FIXME: this should be based on the panel size */
+#define PREFERRED_SIZE 48
+		if (button->pixbuf == NULL)
+			button->pixbuf = get_missing (PREFERRED_SIZE);
+#undef PREFERRED_SIZE
+
 		if (!button->pixbuf)
 			return;
 	}
 
-	if (button->size <= 1)
-		return;
-
 	width  = gdk_pixbuf_get_width  (button->pixbuf);
 	height = gdk_pixbuf_get_height (button->pixbuf);
 
@@ -319,9 +260,17 @@ button_widget_reload_pixbuf (ButtonWidget *button)
 }
 
 static void
-button_widget_theme_changed (ButtonWidget *button)
+button_widget_icon_theme_changed (ButtonWidget *button)
 {
-	button_widget_reload_pixbuf (button);
+	if (button->filename != NULL)
+		button_widget_reload_pixbuf (button);
+}
+
+static void
+button_widget_gtk_theme_changed (ButtonWidget *button)
+{
+	if (button->stock_id != NULL)
+		button_widget_reload_pixbuf (button);
 }
 
 static void
@@ -329,10 +278,6 @@ button_widget_finalize (GObject *object)
 {
 	ButtonWidget *button = (ButtonWidget *) object;
 
-	if (button->pressed_timeout)
-		g_source_remove (button->pressed_timeout);
-	button->pressed_timeout = 0;
-
 	button_widget_unset_pixbufs (button);
 
 	g_free (button->filename);
@@ -357,10 +302,16 @@ button_widget_get_property (GObject    *object,
 	button = BUTTON_WIDGET (object);
 
 	switch (prop_id) {
+	case PROP_ACTIVATABLE:
+		g_value_set_boolean (value, button->activatable);
+		break;
 	case PROP_HAS_ARROW:
 		g_value_set_boolean (value, button->arrow);
 		break;
-	case PROP_ORIENT:
+	case PROP_DND_HIGHLIGHT:
+		g_value_set_boolean (value, button->dnd_highlight);
+		break;
+	case PROP_ORIENTATION:
 		g_value_set_enum (value, button->orientation);
 		break;
 	case PROP_ICON_NAME:
@@ -388,50 +339,24 @@ button_widget_set_property (GObject      *object,
 	button = BUTTON_WIDGET (object);
 
 	switch (prop_id) {
-		const char *icon_name;
-		const char *stock_id;
-
-	case PROP_HAS_ARROW:
-		button->arrow = g_value_get_boolean (value) ? 1 : 0;
-		gtk_widget_queue_draw (GTK_WIDGET (button));
+	case PROP_ACTIVATABLE:
+		button_widget_set_activatable (button, g_value_get_boolean (value));
 		break;
-	case PROP_ORIENT:
-		button->orientation = g_value_get_enum (value);
-		gtk_widget_queue_draw (GTK_WIDGET (button));
+	case PROP_HAS_ARROW:
+		button_widget_set_has_arrow (button, g_value_get_boolean (value));
+		break;
+	case PROP_DND_HIGHLIGHT:
+		button_widget_set_dnd_highlight (button, g_value_get_boolean (value));
+		break;
+	case PROP_ORIENTATION:
+		button_widget_set_orientation (button, g_value_get_enum (value));
 		break;
 	case PROP_ICON_NAME:
-		icon_name = g_value_get_string (value);
-
-		g_assert (!button->filename || !button->stock_id);
-
-		if (button->stock_id) {
-			g_free (button->stock_id);
-			button->stock_id = NULL;
-		}
-
-		if (button->filename)
-			g_free (button->filename);
-		button->filename = g_strdup (icon_name);
-
-		button_widget_reload_pixbuf (button);
+		button_widget_set_icon_name (button, g_value_get_string (value));
 		break;
 	case PROP_STOCK_ID:
-		stock_id = g_value_get_string (value);
-
-		g_assert (!button->filename || !button->stock_id);
-
-		if (button->filename) {
-			g_free (button->filename);
-			button->filename = NULL;
-		}
-
-		if (button->stock_id)
-			g_free (button->stock_id);
-		button->stock_id = g_strdup (stock_id);
-
-		button_widget_reload_pixbuf (button);
+		button_widget_set_stock_id (button, g_value_get_string (value));
 		break;
-
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -446,8 +371,8 @@ get_missing (int preffered_size)
 	GdkPixbuf *retval = NULL;
 
 	if (default_pixmap == NULL)
-		default_pixmap = panel_pixmap_discovery ("gnome-unknown.png",
-							 FALSE /* fallback */);
+		default_pixmap = gnome_desktop_item_find_icon (panel_icon_theme, "gnome-unknown.png",
+							       preffered_size, 0);
 	if (default_pixmap != NULL)
 		retval = gdk_pixbuf_new_from_file (default_pixmap,
 						   NULL);
@@ -466,7 +391,10 @@ load_pixbuf (const char  *file,
 	GdkPixbuf *retval = NULL;
 	char      *full;
 
-	full = gnome_desktop_item_find_icon (panel_icon_theme, file, preffered_size, 0);
+	if (g_path_is_absolute (file))
+		full = g_strdup (file);
+	else
+		full = gnome_desktop_item_find_icon (panel_icon_theme, file, preffered_size, 0);
 
 	if (full) {
 		retval = gdk_pixbuf_new_from_file (full, &error);
@@ -492,87 +420,75 @@ button_load_pixbuf (const char  *file,
 #define PREFERRED_SIZE 48
 
 	GdkPixbuf *retval = NULL;
-	char *tmp;
 
 	if (string_empty (file))
-		return get_missing (PREFERRED_SIZE);
+		return NULL;
 
-	tmp = g_path_get_basename (file);
-	retval = load_pixbuf (tmp, PREFERRED_SIZE, error);
-	g_free (tmp);
+	retval = load_pixbuf (file, PREFERRED_SIZE, NULL);
+	if (!retval) {
+		char *tmp;
 
-	if (!retval && g_path_is_absolute (file)) {
 		if (error && *error) {
 			g_free (*error);
 			*error = NULL;
 		}
 
-		retval = load_pixbuf (file, PREFERRED_SIZE, error);
+		tmp = g_path_get_basename (file);
+		retval = load_pixbuf (tmp, PREFERRED_SIZE, error);
+		g_free (tmp);
 	}
 
-	if (!retval)
-		retval = get_missing (PREFERRED_SIZE);
-
 	return retval;
 
 #undef PREFERRED_SIZE
 }
 
-#define SCALE(x) (((x)*size)/48.0)
-
 static void
 draw_arrow (GdkPoint         *points,
 	    PanelOrientation  orientation,
-	    int               size)
+	    int               width,
+	    int               height)
 {
+	double scale;
+
+	scale = (orientation & PANEL_HORIZONTAL_MASK ? height : width) / 48.0;
+
 	switch (orientation) {
 	case PANEL_ORIENTATION_TOP:
-		points[0].x = SCALE(4);
-		points[0].y = SCALE(48 - 10);
-		points[1].x = SCALE(12);
-		points[1].y = SCALE(48 - 10);
-		points[2].x = SCALE(8);
-		points[2].y = SCALE(48 - 3);
+		points [0].x = 4 * scale;
+		points [0].y = height - 10 * scale;
+		points [1].x = 12 * scale;
+		points [1].y = height - 10 * scale;
+		points [2].x = 8 * scale;
+		points [2].y = height - 3 * scale;
 		break;
 	case PANEL_ORIENTATION_BOTTOM:
-		points[0].x = SCALE(48-12);
-		points[0].y = SCALE(10);
-		points[1].x = SCALE(48-4);
-		points[1].y = SCALE(10);
-		points[2].x = SCALE(48-8);
-		points[2].y = SCALE(3);
+		points [0].x = width - 12 * scale;
+		points [0].y = 10 * scale;
+		points [1].x = width - 4 * scale;
+		points [1].y = 10 * scale;
+		points [2].x = width - 8 * scale;
+		points [2].y = 3 * scale;
 		break;
 	case PANEL_ORIENTATION_LEFT:
-		points[0].x = SCALE(48 - 10);
-		points[0].y = SCALE(48 - 12);
-		points[1].x = SCALE(48 - 10);
-		points[1].y = SCALE(48 - 4);
-		points[2].x = SCALE(48 - 3);
-		points[2].y = SCALE(48 - 8);
+		points [0].x = width  - 10 * scale;
+		points [0].y = height - 12 * scale;
+		points [1].x = width  - 10 * scale;
+		points [1].y = height - 4 * scale;
+		points [2].x = width  - 3 * scale;
+		points [2].y = height - 8 * scale;
 		break;
 	case PANEL_ORIENTATION_RIGHT:
-		points[0].x = SCALE(10);
-		points[0].y = SCALE(4);
-		points[1].x = SCALE(10);
-		points[1].y = SCALE(12);
-		points[2].x = SCALE(3);
-		points[2].y = SCALE(8);
+		points [0].x = 10 * scale;
+		points [0].y = 4  * scale;
+		points [1].x = 10 * scale;
+		points [1].y = 12 * scale;
+		points [2].x = 3  * scale;
+		points [2].y = 8  * scale;
 		break;
 	}
 }
 
-void
-button_widget_set_dnd_highlight(ButtonWidget *button, gboolean highlight)
-{
-	g_return_if_fail (button != NULL);
-	g_return_if_fail (BUTTON_IS_WIDGET (button));
-
-	if(button->dnd_highlight != highlight) {
-		button->dnd_highlight = highlight;
-		gtk_widget_queue_draw (GTK_WIDGET (button));
-	}
-}
-
 static gboolean
 button_widget_expose (GtkWidget         *widget,
 		      GdkEventExpose    *event)
@@ -580,7 +496,7 @@ button_widget_expose (GtkWidget         *widget,
 	ButtonWidget *button_widget;
 	GtkButton *button;
 	GdkRectangle area, image_bound;
-	int off, size;
+	int off;
 	int x, y, w, h;
 	GdkPixbuf *pb = NULL;
   
@@ -590,24 +506,30 @@ button_widget_expose (GtkWidget         *widget,
 	button_widget = BUTTON_WIDGET (widget);
 	button = GTK_BUTTON (widget);
 	
-	if (!GTK_WIDGET_VISIBLE (widget) || !GTK_WIDGET_MAPPED (widget)) {
+	if (!GTK_WIDGET_VISIBLE (widget) || !GTK_WIDGET_MAPPED (widget))
+		return FALSE;
+
+	if (!button_widget->scaled_hc && !button_widget->scaled)
 		return FALSE;
-	}
 
-	size = widget->allocation.height;
 	/* offset for pressed buttons */
-	off = (button->in_button && button->button_down) ?
-		SCALE(BUTTON_WIDGET_DISPLACEMENT) : 0;
-	
-	if (panel_global_config_get_highlight_when_over () && 
+	off = (button_widget->activatable && button->in_button && button->button_down) ?
+		BUTTON_WIDGET_DISPLACEMENT * widget->allocation.height / 48.0 : 0;
+
+	if (!button_widget->activatable) {
+		pb = gdk_pixbuf_copy (button_widget->scaled);
+		gdk_pixbuf_saturate_and_pixelate (button_widget->scaled,
+						  pb,
+						  0.8,
+						  TRUE);
+	} else if (panel_global_config_get_highlight_when_over () && 
 	    (button->in_button || GTK_WIDGET_HAS_FOCUS (widget)))
-		pb = button_widget->scaled_hc;
+		pb = g_object_ref (button_widget->scaled_hc);
 	else
-		pb = button_widget->scaled;
+		pb = g_object_ref (button_widget->scaled);
+
+	g_assert (pb != NULL);
 
-	if (!pb)
-		return FALSE;
-	
 	w = gdk_pixbuf_get_width (pb);
 	h = gdk_pixbuf_get_height (pb);
 	x = widget->allocation.x + off + (widget->allocation.width - w)/2;
@@ -628,11 +550,13 @@ button_widget_expose (GtkWidget         *widget,
 				 image_bound.width, image_bound.height,
 				 GDK_RGB_DITHER_NORMAL,
 				 0, 0);
+
+	g_object_unref (pb);
 	
 	if (button_widget->arrow) {
 		int i;
 		GdkPoint points[3];
-		draw_arrow (points, button_widget->orientation, widget->allocation.height);
+		draw_arrow (points, button_widget->orientation, widget->allocation.width, widget->allocation.height);
 		for (i = 0; i < 3; i++) {
 			points[i].x += off + widget->allocation.x;
 			points[i].y += off + widget->allocation.y;
@@ -704,30 +628,27 @@ button_widget_size_allocate (GtkWidget     *widget,
 
 	if (GTK_WIDGET_REALIZED (widget)) {
 		PanelWidget *panel;
-		int          x, y, w, h;
 
 		panel = PANEL_WIDGET (widget->parent);
 
-		calculate_overlay_geometry (
-			panel, GTK_WIDGET (panel->toplevel),
-			widget, &x, &y, &w, &h);
-
-		gdk_window_move_resize (button->event_window, x, y, w, h);
+		gdk_window_move_resize (button->event_window, 
+					allocation->x,
+					allocation->y,
+					allocation->width,
+					allocation->height);
 	}
 }
 
-static gboolean
-pressed_timeout_func(gpointer data)
+static void
+button_widget_activate (GtkButton *button)
 {
-	ButtonWidget *button;
+	ButtonWidget *button_widget = BUTTON_WIDGET (button);
 
-	g_return_val_if_fail (BUTTON_IS_WIDGET (data), FALSE);
+	if (!button_widget->activatable)
+		return;
 
-	button = BUTTON_WIDGET (data);
-
-	button->pressed_timeout = 0;
-
-	return FALSE;
+	if (GTK_BUTTON_CLASS (parent_class)->activate)
+		GTK_BUTTON_CLASS (parent_class)->activate (button);
 }
 
 static gboolean
@@ -740,7 +661,7 @@ button_widget_button_press (GtkWidget *widget, GdkEventButton *event)
 
 	button = BUTTON_WIDGET (widget);
 
-	if (button->pressed_timeout)
+	if (!button->activatable)
 		return TRUE;
 
 	return GTK_WIDGET_CLASS (parent_class)->button_press_event (widget, event);
@@ -749,10 +670,14 @@ button_widget_button_press (GtkWidget *widget, GdkEventButton *event)
 static gboolean
 button_widget_enter_notify (GtkWidget *widget, GdkEventCrossing *event)
 {
+	gboolean in_button;
+
 	g_return_val_if_fail (BUTTON_IS_WIDGET (widget), FALSE);
 
+	in_button = GTK_BUTTON (widget)->in_button;
 	GTK_WIDGET_CLASS (parent_class)->enter_notify_event (widget, event);
-	if (GTK_BUTTON (widget)->in_button)
+	if (in_button != GTK_BUTTON (widget)->in_button &&
+	    panel_global_config_get_highlight_when_over ())
 		gtk_widget_queue_draw (widget);
 
 	return FALSE;
@@ -774,30 +699,6 @@ button_widget_leave_notify (GtkWidget *widget, GdkEventCrossing *event)
 	return FALSE;
 }
 
-static void
-button_widget_button_pressed (GtkButton *button)
-{
-	ButtonWidget *button_widget;
-
-	g_return_if_fail (BUTTON_IS_WIDGET (button));
-
-	GTK_BUTTON_CLASS (parent_class)->pressed (button);
-
-	button_widget = BUTTON_WIDGET (button);
-	button_widget->pressed_timeout =
-		g_timeout_add (400, pressed_timeout_func, button_widget);
-        gtk_widget_queue_draw (GTK_WIDGET (button));
-}
-
-static void
-button_widget_button_released (GtkButton *button)
-{
-	g_return_if_fail (BUTTON_IS_WIDGET (button));
-
-	GTK_BUTTON_CLASS (parent_class)->released (button);
-        gtk_widget_queue_draw (GTK_WIDGET (button));
-}
-
 static void
 button_widget_instance_init (ButtonWidget *button)
 {
@@ -811,12 +712,15 @@ button_widget_instance_init (ButtonWidget *button)
 	button->ignore_leave  = FALSE;
 	button->dnd_highlight = FALSE;
 
-	button->pressed_timeout = 0;
-	button->size            = 0;
+	button->size = 0;
 
-	panel_signal_connect_object_while_alive (
-			panel_icon_theme, "changed",
-			G_CALLBACK (button_widget_theme_changed), button);
+	g_signal_connect_object (panel_icon_theme, "changed",
+				 G_CALLBACK (button_widget_icon_theme_changed),
+				 button,
+				 G_CONNECT_SWAPPED);
+
+	g_signal_connect (button, "style-set", 
+			  G_CALLBACK (button_widget_gtk_theme_changed), button);
 }
 
 static void
@@ -833,7 +737,7 @@ button_widget_class_init (ButtonWidgetClass *klass)
 	gobject_class->set_property = button_widget_set_property;
 	  
 	widget_class->realize            = button_widget_realize;
-	widget_class->parent_set         = button_widget_parent_set;
+	widget_class->unrealize          = button_widget_unrealize;
 	widget_class->size_allocate      = button_widget_size_allocate;
 	widget_class->size_request       = button_widget_size_request;
 	widget_class->button_press_event = button_widget_button_press;
@@ -841,24 +745,41 @@ button_widget_class_init (ButtonWidgetClass *klass)
 	widget_class->leave_notify_event = button_widget_leave_notify;
 	widget_class->expose_event       = button_widget_expose;
 
-	button_class->pressed  = button_widget_button_pressed;
-	button_class->released = button_widget_button_released;
+	button_class->activate = button_widget_activate;
+
+	g_object_class_install_property (
+			gobject_class,
+			PROP_ACTIVATABLE,
+			g_param_spec_boolean ("activatable",
+					      "Activatable",
+					      "Whether the button is activatable",
+					      TRUE,
+					      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	g_object_class_install_property (
 			gobject_class,
 			PROP_HAS_ARROW,
 			g_param_spec_boolean ("has-arrow",
-					      _("Has Arrow"),
-					      _("Whether or not to draw an arrow indicator"),
+					      "Has Arrow",
+					      "Whether or not to draw an arrow indicator",
 					      FALSE,
 					      G_PARAM_READWRITE));
 
 	g_object_class_install_property (
 			gobject_class,
-			PROP_ORIENT,
-			g_param_spec_enum ("orient",
-					   _("Orientation"),
-					   _("The ButtonWidget orientation"),
+			PROP_DND_HIGHLIGHT,
+			g_param_spec_boolean ("dnd-highlight",
+					      "Drag and drop Highlight",
+					      "Whether or not to highlight the icon during drag and drop",
+					      FALSE,
+					      G_PARAM_READWRITE));
+
+	g_object_class_install_property (
+			gobject_class,
+			PROP_ORIENTATION,
+			g_param_spec_enum ("orientation",
+					   "Orientation",
+					   "The ButtonWidget orientation",
 					   PANEL_TYPE_ORIENTATION,
 					   PANEL_ORIENTATION_TOP,
 					   G_PARAM_READWRITE));
@@ -867,8 +788,8 @@ button_widget_class_init (ButtonWidgetClass *klass)
 			gobject_class,
 			PROP_ICON_NAME,
 			g_param_spec_string ("icon-name",
-					     _("Icon Name"),
-					     _("The desired icon for the ButtonWidget"),
+					     "Icon Name",
+					     "The desired icon for the ButtonWidget",
 					     NULL,
 					     G_PARAM_READWRITE));
 
@@ -876,8 +797,8 @@ button_widget_class_init (ButtonWidgetClass *klass)
 			gobject_class,
 			PROP_STOCK_ID,
 			g_param_spec_string ("stock-id",
-					     _("Stock Icon ID"),
-					     _("The desired stock icon for the ButtonWidget"),
+					     "Stock Icon ID",
+					     "The desired stock icon for the ButtonWidget",
 					     NULL,
 					     G_PARAM_READWRITE));
 }
@@ -917,7 +838,7 @@ button_widget_new (const char       *filename,
 	retval = g_object_new (
 			BUTTON_TYPE_WIDGET,
 			"has-arrow", arrow,
-			"orient", orientation,
+			"orientation", orientation,
 			"icon-name", filename,
 			NULL);
 	
@@ -934,7 +855,7 @@ button_widget_new_from_stock (const char       *stock_id,
 	retval = g_object_new (
 			BUTTON_TYPE_WIDGET,
 			"has-arrow", arrow,
-			"orient", orientation,
+			"orientation", orientation,
 			"stock-id", stock_id,
 			NULL);
 	
@@ -942,12 +863,55 @@ button_widget_new_from_stock (const char       *stock_id,
 }
 
 void
-button_widget_set_pixmap (ButtonWidget *button,
-			  const char   *pixmap)
+button_widget_set_activatable (ButtonWidget *button,
+			       gboolean      activatable)
 {
 	g_return_if_fail (BUTTON_IS_WIDGET (button));
 
-	g_object_set (G_OBJECT (button), "icon-name", pixmap, NULL);
+	activatable = activatable != FALSE;
+
+	if (button->activatable != activatable) {
+		button->activatable = activatable;
+
+		if (GTK_WIDGET_DRAWABLE (button))
+			gtk_widget_queue_draw (GTK_WIDGET (button));
+
+		g_object_notify (G_OBJECT (button), "activatable");
+	}
+}
+
+gboolean
+button_widget_get_activatable (ButtonWidget *button)
+{
+	g_return_val_if_fail (BUTTON_IS_WIDGET (button), FALSE);
+
+	return button->activatable;
+}
+
+void
+button_widget_set_icon_name (ButtonWidget *button,
+			     const char   *icon_name)
+{
+	g_return_if_fail (BUTTON_IS_WIDGET (button));
+
+	if (button->filename && icon_name && !strcmp (button->filename, icon_name))
+		return;
+
+	if (button->filename)
+		g_free (button->filename);
+	button->filename = g_strdup (icon_name);
+
+	button_widget_reload_pixbuf (button);
+
+	g_object_notify (G_OBJECT (button), "icon-name");
+}
+
+const char *
+button_widget_get_icon_name (ButtonWidget *button)
+{
+	g_return_val_if_fail (BUTTON_IS_WIDGET (button), NULL);
+
+	return button->filename;
 }
 
 void
@@ -955,17 +919,102 @@ button_widget_set_stock_id (ButtonWidget *button,
 			    const char   *stock_id)
 {
 	g_return_if_fail (BUTTON_IS_WIDGET (button));
+	
+	if (button->stock_id && stock_id && !strcmp (button->stock_id, stock_id))
+		return;
 
-	g_object_set (G_OBJECT (button), "stock-id", stock_id, NULL);
+	if (button->stock_id)
+		g_free (button->stock_id);
+	button->stock_id = g_strdup (stock_id);
+
+	button_widget_reload_pixbuf (button);
+
+	g_object_notify (G_OBJECT (button), "stock-id");
+}
+
+const char *
+button_widget_get_stock_id (ButtonWidget *button)
+{
+	g_return_val_if_fail (BUTTON_IS_WIDGET (button), NULL);
+	
+	return button->stock_id;
 }
 
 void
-button_widget_set_params (ButtonWidget     *button,
-			  gboolean          arrow,
-			  PanelOrientation  orientation)
+button_widget_set_orientation (ButtonWidget     *button,
+			       PanelOrientation  orientation)
 {
 	g_return_if_fail (BUTTON_IS_WIDGET (button));
 
-	g_object_set (G_OBJECT (button), "has-arrow", arrow, NULL);
-	g_object_set (G_OBJECT (button), "orient", orientation, NULL);
+	if (button->orientation == orientation)
+		return;
+
+	button->orientation = orientation;
+
+	/* Force a re-scale */
+	button->size = -1;
+
+	gtk_widget_queue_resize (GTK_WIDGET (button));
+
+	g_object_notify (G_OBJECT (button), "orientation");
+}
+
+PanelOrientation
+button_widget_get_orientation (ButtonWidget *button)
+{
+	g_return_val_if_fail (BUTTON_IS_WIDGET (button), 0);
+
+	return button->orientation;
+}
+
+void
+button_widget_set_has_arrow (ButtonWidget *button,
+			     gboolean      has_arrow)
+{
+	g_return_if_fail (BUTTON_IS_WIDGET (button));
+
+	has_arrow = has_arrow != FALSE;
+
+	if (button->arrow == has_arrow)
+		return;
+
+	button->arrow = has_arrow;
+
+	gtk_widget_queue_draw (GTK_WIDGET (button));
+
+	g_object_notify (G_OBJECT (button), "has-arrow");
+}
+
+gboolean
+button_widget_get_has_arrow (ButtonWidget *button)
+{
+	g_return_val_if_fail (BUTTON_IS_WIDGET (button), FALSE);
+
+	return button->arrow;
+}
+
+void
+button_widget_set_dnd_highlight (ButtonWidget *button,
+				 gboolean      dnd_highlight)
+{
+	g_return_if_fail (BUTTON_IS_WIDGET (button));
+
+	dnd_highlight = dnd_highlight != FALSE;
+
+	if (button->dnd_highlight == dnd_highlight)
+		return;
+
+	button->dnd_highlight = dnd_highlight;
+
+	gtk_widget_queue_draw (GTK_WIDGET (button));
+
+	g_object_notify (G_OBJECT (button), "dnd-highlight");
+}
+
+gboolean
+button_widget_get_dnd_highlight (ButtonWidget *button)
+{
+	g_return_val_if_fail (BUTTON_IS_WIDGET (button), FALSE);
+
+	return button->dnd_highlight;
 }
