@@ -53,6 +53,7 @@
 #include "panel-multiscreen.h"
 #include "menu.h"
 #include "menu-fentry.h"
+#include "panel-lockdown.h"
 
 #define ICON_SIZE 48.0
 
@@ -421,7 +422,7 @@ panel_run_dialog_response (PanelRunDialog *dialog,
 		break;
 	case GTK_RESPONSE_HELP:
 		panel_show_help (gtk_window_get_screen (GTK_WINDOW (run_dialog)),
-				 "wgoseditmainmenu.xml", "gospanel-23");
+				 "user-guide.xml", "gospanel-23");
 		break;
 	default:
 		break;
@@ -948,10 +949,11 @@ panel_run_dialog_update_content (PanelRunDialog *dialog,
 		if (dialog->list_expander->parent)
 			gtk_container_remove (GTK_CONTAINER (dialog->list_expander->parent),
 					      dialog->list_expander);
+
+		gtk_window_set_resizable (GTK_WINDOW (dialog->run_dialog), FALSE);
+                gtk_widget_grab_focus (dialog->gtk_entry);
 		
-	}
-	
-	if (show_list) {
+	} else if (show_list) {
 		gtk_window_resize (GTK_WINDOW (dialog->run_dialog), 100, 300);
 		gtk_window_set_resizable (GTK_WINDOW (dialog->run_dialog), TRUE);
 		gtk_widget_grab_focus (dialog->program_list);
@@ -1469,7 +1471,7 @@ panel_run_dialog_present (GdkScreen *screen)
 	GladeXML *gui;
 	char     *glade_file;
 
-	if (panel_profile_get_inhibit_command_line ())
+	if (panel_lockdown_get_disable_command_line ())
 		return;
 
 	if (static_dialog) {
